using System;
using System.IO;

public class RPSGame {

  /// <summary>
  /// RPSGame (File version)    Horspool and Bishop    December 2002
  /// Keeps track of state of the Rock-Paper-Scissors game.
  /// </summary>
  public const int Rock = 0;
  public const int Paper = 1;
  public const int Scissors = 2;
  string[] MoveNames = {"Rock", "Scissors", "Paper"}; 
  const string hFileName = "C:\\WINDOWS\\Temp\\rpsdata.txt";
  Random r;
  int computerChoice, userChoice;
  int numWins, numLosses, numDraws;
  int[,] WinMatrix = { {0,-1,1}, {1,0,-1}, {-1,1,0} };
  int[,,] Context2 = new int[3,3,3];

  public string ComputersChoice {
    get { MakeRandomChoice(); return MoveNames[computerChoice];} }

  public int    Wins   { get { return numWins; } }
  public int    Losses { get { return numLosses; } }
  public int    Draws  { get { return numDraws; } }

  public RPSGame() {
    r = new Random();
    LoadHistoryFile();
  }

  public void EndRPSGame() {
    SaveHistoryFile();
  }

  void LoadHistoryFile() {
    try {
      StreamReader hfile = new StreamReader(hFileName);
      // read in the entire history file as one string
      string contents = hfile.ReadToEnd();
      hfile.Close();
      // split it into separate strings
      string[] nums = contents.Split();
      int ix = 0;
      for( int i=0; i<3; i++ ) {
        for( int j=0; j<3; j++ ) {
          for( int k=0; k<3; k++ ) {
            while(nums[ix] == "") ix++;
              Context2[i,j,k] = int.Parse(nums[ix++]);
          } 
        } 
      } 
      while(nums[ix] == "") ix++;
      computerChoice = int.Parse(nums[ix++]);
      while(nums[ix] == "") ix++;
      userChoice = int.Parse(nums[ix++]);
    } catch(Exception) {
      // the history file did not exist or did not
      // contain data in the expected format
      for( int i=0; i<3; i++ )
        for( int j=0; j<3; j++ )
          for( int k=0; k<3; k++ )
            Context2[i,j,k] = 1;
    } 
    computerChoice = r.Next(3);
    userChoice     = r.Next(3);
  }

  void SaveHistoryFile() {
    try {
      StreamWriter hfile = new StreamWriter(hFileName);
      for( int i=0; i<3; i++ ) {
        for( int j=0; j<3; j++ ) {
          for( int k=0; k<3; k++ )
            hfile.Write("{0} ", Context2[i,j,k]);
        } 
        hfile.WriteLine();
      } 
      hfile.WriteLine("{0} {1}", computerChoice, userChoice);
      hfile.Close();
    } catch(Exception) {
      Console.WriteLine("Unable to write to {0}", hFileName);
    }
  }

  public void MakeRandomChoice() {
    int rcount = Context2[userChoice,computerChoice,Rock];
    int pcount = Context2[userChoice,computerChoice,Paper];
    int scount = Context2[userChoice,computerChoice,Scissors];
    double total = rcount+pcount+scount;
    double d = r.NextDouble();
    if (d < rcount/total) {
      computerChoice = Paper;
    } else if (d < (rcount+pcount)/total) {
      computerChoice = Scissors;
    } else {
      computerChoice = Rock;
    } 
  }

  public string ComparePlays(string userPlay) {
    ///<summary>
    /// Determines whether the userPlay beats the computer.
    /// The result is a string "win", "lose" or "draw"
    ///</summary>
    int prevUserChoice = userChoice;
    userChoice = MoveToInt(userPlay);
    Context2[prevUserChoice,computerChoice,userChoice]++; 
    int result = WinMatrix[userChoice, computerChoice];
    switch (result) {
    case -1:  numLosses++;  return "lose";
    case 1:   numWins++;    return "win";
    default:  numDraws++;   return "draw";
    } 
  } 

  int MoveToInt( string move ) {
    switch(move.ToLower()) {
    case "rock":  return Rock;
    case "paper": return Paper;
    case "scissors": return Scissors;
    default:
      throw new Exception("Unrecognized move: "+move);
    }
  }
}

